# -*- coding: utf-8 -*-
"""

    mslib.msui.kmloverlay_dockwidget
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Control widget to configure remote sensing overlays.

    This file is part of mss.

    :copyright: Copyright 2017 Joern Ungermann
    :copyright: Copyright 2017-2020 by the mss team, see AUTHORS.
    :license: APACHE-2.0, see LICENSE for details.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import copy
from fs import open_fs
import logging
from lxml import etree, objectify
import os
from matplotlib import patheffects

from mslib.msui.mss_qt import QtGui, QtWidgets, get_open_filename
from mslib.msui.mss_qt import ui_kmloverlay_dockwidget as ui
from mslib.utils import save_settings_qsettings, load_settings_qsettings


KMLPARSER = objectify.makeparser(strip_cdata=False)


class KMLPatch(object):
    """
    Represents a KML overlay.

    KML overlay implementation is currently very crude and basic and most features are not supported.
    """

    def __init__(self, mapcanvas, kml, overwrite=False, color="red", linewidth=1):
        self.map = mapcanvas
        self.kml = kml
        self.patches = []
        self.color = color
        self.linewidth = linewidth
        self.overwrite = overwrite
        self.draw()

    def compute_xy(self, coordinates):
        coords = str(coordinates).split()
        lons, lats = [[float(_x.split(",")[_i]) for _x in coords] for _i in range(2)]
        return self.map(lons, lats)

    def add_polygon(self, polygon, style, _):
        """
        Plot KML polygons

        :param polygon: pykml object specifying a polygon
        """
        kwargs = style.get("LineStyle", {"linewidth": self.linewidth, "color": self.color})
        for boundary in ["outerBoundaryIs", "innerBoundaryIs"]:
            if hasattr(polygon, boundary):
                x, y = self.compute_xy(getattr(polygon, boundary).LinearRing.coordinates)
                self.patches.append(self.map.plot(x, y, "-", zorder=10, **kwargs))

    def add_point(self, point, style, name):
        """
        Plot KML point

        :param point: pykml object specifying point
        :param name: name of placemark for annotation
        """
        x, y = self.compute_xy(point.coordinates)
        self.patches.append(self.map.plot(x[0], y[0], "o", zorder=10, color=self.color))
        if name is not None:
            self.patches.append([self.map.ax.annotate(
                name, xy=(x[0], y[0]), xycoords="data", xytext=(5, 5), textcoords='offset points', zorder=10,
                path_effects=[patheffects.withStroke(linewidth=2, foreground='w')])])

    def add_line(self, line, style, _):
        """
        Plot KML line

        :param line: pykml LineString object
        """
        kwargs = style.get("LineStyle", {"linewidth": self.linewidth, "color": self.color})
        x, y = self.compute_xy(line.coordinates)
        self.patches.append(self.map.plot(x, y, "-", zorder=10, **kwargs))

    def parse_geometries(self, placemark):
        name = getattr(placemark, "name", None)
        styleurl = str(getattr(placemark, "styleUrl", ""))
        if styleurl and len(styleurl) > 0 and styleurl[0] == "#":
            # Remove # at beginning of style marking a locally defined style.
            # general urls for styles are not supported
            styleurl = styleurl[1:]
        style = self.parse_local_styles(
            placemark, self.styles.get(styleurl, {}))
        for attr_name, method in (
                ("Point", self.add_point),
                ("Polygon", self.add_polygon),
                ("LineString", self.add_line),
                ("MultiGeometry", self.parse_geometries)):
            for attr in getattr(placemark, attr_name, []):
                logging.debug("Found %s", attr_name)
                method(attr, style, name)

    def parse_placemarks(self, level):
        for placemark in getattr(level, "Placemark", []):
            name = getattr(placemark, "name", None)
            logging.debug("Placemark: %s", name)
            self.parse_geometries(placemark)
        for folder in getattr(level, "Folder", []):
            name = getattr(folder, "name", None)
            logging.debug("Folder: %s", name)
            self.parse_placemarks(folder)

    def get_style_params(self, style, color=None, linewidth=None):
        if color is None:
            color = self.color
        if linewidth is None:
            linewidth = self.linewidth
        result = {
            "color": str(getattr(style, "color", "")),
            "linewidth": float(getattr(style, "width", linewidth))
        }
        logging.debug("color before %s", result["color"])
        if len(result["color"]) == 7 and result["color"][0] == "#":
            result["color"] = [(int(result["color"][i:i + 2], 16) / 255.) for i in range(1, 8, 2)]
        elif len(result["color"]) == 8:
            result["color"] = [(int(result["color"][i:i + 2], 16) / 255.) for i in range(0, 8, 2)][::-1]
        else:
            result["color"] = color
        logging.debug("color after %s", result["color"])
        return result

    def parse_styles(self, level):
        for style in getattr(level, "Style", []):
            name = style.attrib.get("id")
            if name is None:
                continue
            self.styles[name] = {
                "LineStyle": self.get_style_params(getattr(style, "LineStyle", None)),
                "PolyStyle": self.get_style_params(getattr(style, "PolyStyle", None)),
            }
        for folder in getattr(level, "Folder", []):
            name = getattr(folder, "name", None)
            logging.debug("Folder: %s", name)
            self.parse_styles(folder)

    def parse_local_styles(self, level, default_styles):
        logging.debug("styles before %s", default_styles)
        local_styles = copy.deepcopy(default_styles)
        for style in getattr(level, "Style", []):
            logging.debug("style %s", style)
            for supported in ["LineStyle", "PolyStyle"]:
                if supported in local_styles and hasattr(style, supported):
                    local_styles["LineStyle"] = self.get_style_params(
                        getattr(style, supported),
                        color=local_styles[supported]["color"], linewidth=local_styles[supported]["linewidth"])
                elif hasattr(style, supported):
                    local_styles["LineStyle"] = self.get_style_params(getattr(style, supported))
        logging.debug("styles after %s", local_styles)
        return local_styles

    def draw(self):
        """Do the actual plotting of the patch.
        """
        # Plot satellite track.
        self.styles = {}
        if not self.overwrite:
            self.parse_styles(self.kml.Document)
        self.parse_placemarks(self.kml.Document)

        self.map.ax.figure.canvas.draw()

    def update(self, overwrite=None, color=None, linewidth=None):
        """Removes the current plot of the patch and redraws the patch.
           This is necessary, for instance, when the map projection and/or
           extent has been changed.
        """
        if overwrite is not None:
            self.overwrite = overwrite
        if color is not None:
            self.color = color
        if linewidth is not None:
            self.linewidth = linewidth
        self.remove()
        self.draw()

    def remove(self):
        """Remove this satellite patch from the map canvas.
        """
        for patch in self.patches:
            for element in patch:
                element.remove()
        self.patches = []
        self.map.ax.figure.canvas.draw()


class KMLOverlayControlWidget(QtWidgets.QWidget, ui.Ui_KMLOverlayDockWidget):
    """
    This class provides the interface for accessing KML files and
    adding the appropriate patches to the TopView canvas.
    """

    def __init__(self, parent=None, view=None):
        super(KMLOverlayControlWidget, self).__init__(parent)
        self.setupUi(self)
        self.view = view
        self.kml = None
        self.patch = None

        # Connect slots and signals.
        self.btSelectFile.clicked.connect(self.select_file)
        self.btLoadFile.clicked.connect(self.load_file)
        self.pbSelectColour.clicked.connect(self.select_colour)
        self.cbOverlay.stateChanged.connect(self.update_settings)
        self.dsbLineWidth.valueChanged.connect(self.update_settings)
        self.cbManualStyle.stateChanged.connect(self.update_settings)

        self.cbOverlay.setChecked(True)
        self.cbOverlay.setEnabled(False)
        self.cbManualStyle.setChecked(False)

        self.settings_tag = "kmldock"
        settings = load_settings_qsettings(
            self.settings_tag, {"filename": "", "linewidth": 1, "colour": (0, 0, 0, 1)})

        self.leFile.setText(settings["filename"])
        self.dsbLineWidth.setValue(settings["linewidth"])

        palette = QtGui.QPalette(self.pbSelectColour.palette())
        colour = QtGui.QColor()
        colour.setRgbF(*settings["colour"])
        palette.setColor(QtGui.QPalette.Button, colour)
        self.pbSelectColour.setPalette(palette)

    def __del__(self):
        settings = {
            "filename": str(self.leFile.text()),
            "linewidth": self.dsbLineWidth.value(),
            "colour": self.get_color()
        }
        save_settings_qsettings(self.settings_tag, settings)

    def get_color(self):
        button = self.pbSelectColour
        return QtGui.QPalette(button.palette()).color(QtGui.QPalette.Button).getRgbF()

    def update_settings(self):
        """
        Called when the visibility checkbox is toggled and hides/shows
        the overlay if loaded.
        """
        if self.view is not None and self.cbOverlay.isChecked() and self.patch is not None:
            self.view.plot_kml(self.patch)
            self.patch.update(self.cbManualStyle.isChecked(), self.get_color(), self.dsbLineWidth.value())
        elif self.patch is not None:
            self.view.plot_kml(None)

    def select_colour(self):
        button = self.pbSelectColour

        palette = QtGui.QPalette(button.palette())
        colour = palette.color(QtGui.QPalette.Button)
        colour = QtWidgets.QColorDialog.getColor(colour)
        if colour.isValid():
            palette.setColor(QtGui.QPalette.Button, colour)
            button.setPalette(palette)
        self.update_settings()

    def select_file(self):
        """Slot that opens a file dialog to choose a kml file
        """
        filename = get_open_filename(
            self, "Open KML Polygonal File", os.path.dirname(str(self.leFile.text())), "KML Files (*.kml)")
        if not filename:
            return
        self.leFile.setText(filename)

    def load_file(self):
        """
        Loads an KML file selected by the leFile box and constructs the
        corresponding patch.
        """
        _dirname, _name = os.path.split(self.leFile.text())
        _fs = open_fs(_dirname)
        if self.patch is not None:
            self.patch.remove()
            self.view.plot_kml(None)
            self.patch = None
            self.cbOverlay.setEnabled(False)
        try:
            with _fs.open(_name, 'r') as kmlf:
                self.kml = objectify.parse(kmlf, parser=KMLPARSER).getroot()
                self.patch = KMLPatch(self.view.map, self.kml,
                                      self.cbManualStyle.isChecked(), self.get_color(), self.dsbLineWidth.value())
            self.cbOverlay.setEnabled(True)
            if self.view is not None and self.cbOverlay.isChecked():
                self.view.plot_kml(self.patch)
        except (IOError, etree.XMLSyntaxError) as ex:
            logging.error("KML Overlay - %s: %s", type(ex), ex)
            QtWidgets.QMessageBox.critical(
                self, self.tr("KML Overlay"), self.tr("ERROR:\n{}\n{}".format(type(ex), ex)))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'mslib/msui/ui/ui_mscolab_window.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MSSMscolabWindow(object):
    def setupUi(self, MSSMscolabWindow):
        MSSMscolabWindow.setObjectName("MSSMscolabWindow")
        MSSMscolabWindow.resize(510, 510)
        MSSMscolabWindow.setMinimumSize(QtCore.QSize(510, 510))
        self.centralwidget = QtWidgets.QWidget(MSSMscolabWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.groupBox = QtWidgets.QGroupBox(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setMinimumSize(QtCore.QSize(500, 500))
        self.groupBox.setFlat(True)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.urlWidget = QtWidgets.QWidget(self.groupBox)
        self.urlWidget.setObjectName("urlWidget")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.urlWidget)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_2 = QtWidgets.QLabel(self.urlWidget)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_5.addWidget(self.label_2)
        self.url = QtWidgets.QComboBox(self.urlWidget)
        self.url.setObjectName("url")
        self.horizontalLayout_5.addWidget(self.url)
        self.connectMscolab = QtWidgets.QPushButton(self.urlWidget)
        self.connectMscolab.setObjectName("connectMscolab")
        self.horizontalLayout_5.addWidget(self.connectMscolab)
        self.verticalLayout_9.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.status = QtWidgets.QLabel(self.urlWidget)
        self.status.setObjectName("status")
        self.horizontalLayout_6.addWidget(self.status)
        self.disconnectMscolab = QtWidgets.QPushButton(self.urlWidget)
        self.disconnectMscolab.setObjectName("disconnectMscolab")
        self.horizontalLayout_6.addWidget(self.disconnectMscolab)
        spacerItem = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem)
        self.verticalLayout_9.addLayout(self.horizontalLayout_6)
        self.verticalLayout.addWidget(self.urlWidget)
        self.loggedInWidget = QtWidgets.QWidget(self.groupBox)
        self.loggedInWidget.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.loggedInWidget.sizePolicy().hasHeightForWidth())
        self.loggedInWidget.setSizePolicy(sizePolicy)
        self.loggedInWidget.setObjectName("loggedInWidget")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout(self.loggedInWidget)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label = QtWidgets.QLabel(self.loggedInWidget)
        self.label.setMaximumSize(QtCore.QSize(300, 16777215))
        self.label.setObjectName("label")
        self.horizontalLayout_7.addWidget(self.label)
        self.logoutButton = QtWidgets.QPushButton(self.loggedInWidget)
        self.logoutButton.setMaximumSize(QtCore.QSize(60, 16777215))
        self.logoutButton.setObjectName("logoutButton")
        self.horizontalLayout_7.addWidget(self.logoutButton)
        spacerItem1 = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem1)
        self.verticalLayout.addWidget(self.loggedInWidget)
        self.loginWidget = QtWidgets.QWidget(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.loginWidget.sizePolicy().hasHeightForWidth())
        self.loginWidget.setSizePolicy(sizePolicy)
        self.loginWidget.setObjectName("loginWidget")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.loginWidget)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.emailid = QtWidgets.QLineEdit(self.loginWidget)
        self.emailid.setObjectName("emailid")
        self.horizontalLayout_3.addWidget(self.emailid)
        self.password = QtWidgets.QLineEdit(self.loginWidget)
        self.password.setObjectName("password")
        self.horizontalLayout_3.addWidget(self.password)
        self.loginButton = QtWidgets.QPushButton(self.loginWidget)
        self.loginButton.setObjectName("loginButton")
        self.horizontalLayout_3.addWidget(self.loginButton)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem2)
        self.verticalLayout_8.addLayout(self.horizontalLayout_3)
        self.addUser = QtWidgets.QPushButton(self.loginWidget)
        self.addUser.setMaximumSize(QtCore.QSize(80, 16777215))
        self.addUser.setObjectName("addUser")
        self.verticalLayout_8.addWidget(self.addUser)
        self.verticalLayout.addWidget(self.loginWidget)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.listProjects = QtWidgets.QListWidget(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listProjects.sizePolicy().hasHeightForWidth())
        self.listProjects.setSizePolicy(sizePolicy)
        self.listProjects.setAlternatingRowColors(False)
        self.listProjects.setTextElideMode(QtCore.Qt.ElideNone)
        self.listProjects.setObjectName("listProjects")
        self.horizontalLayout_2.addWidget(self.listProjects)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.addProject = QtWidgets.QPushButton(self.groupBox)
        self.addProject.setObjectName("addProject")
        self.verticalLayout_2.addWidget(self.addProject)
        self.export_2 = QtWidgets.QPushButton(self.groupBox)
        self.export_2.setObjectName("export_2")
        self.verticalLayout_2.addWidget(self.export_2)
        self.autoSave = QtWidgets.QCheckBox(self.groupBox)
        self.autoSave.setObjectName("autoSave")
        self.verticalLayout_2.addWidget(self.autoSave)
        self.save_ft = QtWidgets.QPushButton(self.groupBox)
        self.save_ft.setObjectName("save_ft")
        self.verticalLayout_2.addWidget(self.save_ft)
        self.fetch_ft = QtWidgets.QPushButton(self.groupBox)
        self.fetch_ft.setObjectName("fetch_ft")
        self.verticalLayout_2.addWidget(self.fetch_ft)
        self.autosaveStatus = QtWidgets.QLabel(self.groupBox)
        self.autosaveStatus.setText("")
        self.autosaveStatus.setObjectName("autosaveStatus")
        self.verticalLayout_2.addWidget(self.autosaveStatus)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem3)
        self.horizontalLayout_2.addLayout(self.verticalLayout_2)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.tableview = QtWidgets.QPushButton(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tableview.sizePolicy().hasHeightForWidth())
        self.tableview.setSizePolicy(sizePolicy)
        self.tableview.setObjectName("tableview")
        self.horizontalLayout.addWidget(self.tableview)
        self.sideview = QtWidgets.QPushButton(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sideview.sizePolicy().hasHeightForWidth())
        self.sideview.setSizePolicy(sizePolicy)
        self.sideview.setObjectName("sideview")
        self.horizontalLayout.addWidget(self.sideview)
        self.topview = QtWidgets.QPushButton(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.topview.sizePolicy().hasHeightForWidth())
        self.topview.setSizePolicy(sizePolicy)
        self.topview.setObjectName("topview")
        self.horizontalLayout.addWidget(self.topview)
        self.projWindow = QtWidgets.QPushButton(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.projWindow.sizePolicy().hasHeightForWidth())
        self.projWindow.setSizePolicy(sizePolicy)
        self.projWindow.setObjectName("projWindow")
        self.horizontalLayout.addWidget(self.projWindow)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem4)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_4.addWidget(self.groupBox)
        MSSMscolabWindow.setCentralWidget(self.centralwidget)
        self.actionCloseWindow = QtWidgets.QAction(MSSMscolabWindow)
        self.actionCloseWindow.setObjectName("actionCloseWindow")
        MSSMscolabWindow.addAction(self.actionCloseWindow)

        self.retranslateUi(MSSMscolabWindow)
        self.actionCloseWindow.triggered.connect(MSSMscolabWindow.close)
        QtCore.QMetaObject.connectSlotsByName(MSSMscolabWindow)

    def retranslateUi(self, MSSMscolabWindow):
        _translate = QtCore.QCoreApplication.translate
        MSSMscolabWindow.setWindowTitle(_translate("MSSMscolabWindow", "Mscolab Create Projects"))
        self.groupBox.setTitle(_translate("MSSMscolabWindow", "Project listing"))
        self.label_2.setText(_translate("MSSMscolabWindow", "Mscolab URL"))
        self.connectMscolab.setText(_translate("MSSMscolabWindow", "connect"))
        self.status.setText(_translate("MSSMscolabWindow", "Status: disconnected"))
        self.disconnectMscolab.setText(_translate("MSSMscolabWindow", "disconnect"))
        self.label.setText(_translate("MSSMscolabWindow", "TextLabel"))
        self.logoutButton.setText(_translate("MSSMscolabWindow", "logout"))
        self.emailid.setPlaceholderText(_translate("MSSMscolabWindow", "emailid"))
        self.password.setPlaceholderText(_translate("MSSMscolabWindow", "password"))
        self.loginButton.setText(_translate("MSSMscolabWindow", "login"))
        self.addUser.setText(_translate("MSSMscolabWindow", "add user"))
        self.listProjects.setToolTip(_translate("MSSMscolabWindow", "List of mscolab projects."))
        self.addProject.setText(_translate("MSSMscolabWindow", "add project"))
        self.export_2.setText(_translate("MSSMscolabWindow", "export"))
        self.autoSave.setText(_translate("MSSMscolabWindow", "autosave"))
        self.save_ft.setText(_translate("MSSMscolabWindow", "save"))
        self.fetch_ft.setText(_translate("MSSMscolabWindow", "fetch"))
        self.tableview.setText(_translate("MSSMscolabWindow", "tableview"))
        self.sideview.setText(_translate("MSSMscolabWindow", "sideview"))
        self.topview.setText(_translate("MSSMscolabWindow", "topview"))
        self.projWindow.setText(_translate("MSSMscolabWindow", "chat window"))
        self.actionCloseWindow.setText(_translate("MSSMscolabWindow", "CloseWindow"))
        self.actionCloseWindow.setShortcut(_translate("MSSMscolabWindow", "Ctrl+W"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui_trajectories_window.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_TrajectoriesWindow(object):
    def setupUi(self, TrajectoriesWindow):
        TrajectoriesWindow.setObjectName("TrajectoriesWindow")
        TrajectoriesWindow.resize(732, 725)
        self.centralwidget = QtWidgets.QWidget(TrajectoriesWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabGUI = QtWidgets.QTabWidget(self.centralwidget)
        self.tabGUI.setEnabled(True)
        self.tabGUI.setTabPosition(QtWidgets.QTabWidget.North)
        self.tabGUI.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.tabGUI.setObjectName("tabGUI")
        self.tabData = QtWidgets.QWidget()
        self.tabData.setObjectName("tabData")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.tabData)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.groupBox = QtWidgets.QGroupBox(self.tabData)
        self.groupBox.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.groupBox.setFlat(True)
        self.groupBox.setCheckable(False)
        self.groupBox.setObjectName("groupBox")
        self._5 = QtWidgets.QVBoxLayout(self.groupBox)
        self._5.setObjectName("_5")
        self._6 = QtWidgets.QHBoxLayout()
        self._6.setObjectName("_6")
        self.label_4 = QtWidgets.QLabel(self.groupBox)
        self.label_4.setObjectName("label_4")
        self._6.addWidget(self.label_4)
        self.leSelectionQuery = QtWidgets.QLineEdit(self.groupBox)
        self.leSelectionQuery.setObjectName("leSelectionQuery")
        self._6.addWidget(self.leSelectionQuery)
        self.cbSelectElements = QtWidgets.QComboBox(self.groupBox)
        self.cbSelectElements.setObjectName("cbSelectElements")
        self.cbSelectElements.addItem("")
        self.cbSelectElements.addItem("")
        self._6.addWidget(self.cbSelectElements)
        self.btSelectMapElements = QtWidgets.QPushButton(self.groupBox)
        self.btSelectMapElements.setObjectName("btSelectMapElements")
        self._6.addWidget(self.btSelectMapElements)
        self._5.addLayout(self._6)
        self.tvVisibleElements = QtWidgets.QTreeView(self.groupBox)
        self.tvVisibleElements.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.tvVisibleElements.setSortingEnabled(False)
        self.tvVisibleElements.setObjectName("tvVisibleElements")
        self._5.addWidget(self.tvVisibleElements)
        self.verticalLayout_3.addWidget(self.groupBox)
        self.groupBox_7 = QtWidgets.QGroupBox(self.tabData)
        self.groupBox_7.setFlat(True)
        self.groupBox_7.setObjectName("groupBox_7")
        self._7 = QtWidgets.QHBoxLayout(self.groupBox_7)
        self._7.setObjectName("_7")
        self._8 = QtWidgets.QVBoxLayout()
        self._8.setObjectName("_8")
        self._9 = QtWidgets.QHBoxLayout()
        self._9.setObjectName("_9")
        self.label = QtWidgets.QLabel(self.groupBox_7)
        self.label.setMinimumSize(QtCore.QSize(60, 0))
        self.label.setMaximumSize(QtCore.QSize(70, 16777215))
        self.label.setObjectName("label")
        self._9.addWidget(self.label)
        self.cbColour = QtWidgets.QComboBox(self.groupBox_7)
        self.cbColour.setMinimumSize(QtCore.QSize(80, 0))
        self.cbColour.setObjectName("cbColour")
        self.cbColour.addItem("")
        self.cbColour.addItem("")
        self.cbColour.addItem("")
        self.cbColour.addItem("")
        self.cbColour.addItem("")
        self.cbColour.addItem("")
        self.cbColour.addItem("")
        self.cbColour.addItem("")
        self.cbColour.addItem("")
        self._9.addWidget(self.cbColour)
        self.btColour = QtWidgets.QPushButton(self.groupBox_7)
        self.btColour.setMinimumSize(QtCore.QSize(60, 0))
        self.btColour.setMaximumSize(QtCore.QSize(60, 16777215))
        self.btColour.setObjectName("btColour")
        self._9.addWidget(self.btColour)
        self._8.addLayout(self._9)
        self._10 = QtWidgets.QHBoxLayout()
        self._10.setObjectName("_10")
        self.label_2 = QtWidgets.QLabel(self.groupBox_7)
        self.label_2.setMinimumSize(QtCore.QSize(60, 0))
        self.label_2.setMaximumSize(QtCore.QSize(70, 16777215))
        self.label_2.setObjectName("label_2")
        self._10.addWidget(self.label_2)
        self.cbLineStyle = QtWidgets.QComboBox(self.groupBox_7)
        self.cbLineStyle.setMinimumSize(QtCore.QSize(80, 0))
        self.cbLineStyle.setObjectName("cbLineStyle")
        self.cbLineStyle.addItem("")
        self.cbLineStyle.addItem("")
        self.cbLineStyle.addItem("")
        self.cbLineStyle.addItem("")
        self.cbLineStyle.addItem("")
        self._10.addWidget(self.cbLineStyle)
        self.btLineStyle = QtWidgets.QPushButton(self.groupBox_7)
        self.btLineStyle.setMinimumSize(QtCore.QSize(60, 0))
        self.btLineStyle.setMaximumSize(QtCore.QSize(60, 16777215))
        self.btLineStyle.setObjectName("btLineStyle")
        self._10.addWidget(self.btLineStyle)
        self._8.addLayout(self._10)
        self._11 = QtWidgets.QHBoxLayout()
        self._11.setObjectName("_11")
        self.label_3 = QtWidgets.QLabel(self.groupBox_7)
        self.label_3.setMinimumSize(QtCore.QSize(60, 0))
        self.label_3.setMaximumSize(QtCore.QSize(70, 16777215))
        self.label_3.setObjectName("label_3")
        self._11.addWidget(self.label_3)
        self.sbLineWidth = QtWidgets.QSpinBox(self.groupBox_7)
        self.sbLineWidth.setMinimumSize(QtCore.QSize(80, 0))
        self.sbLineWidth.setMinimum(1)
        self.sbLineWidth.setObjectName("sbLineWidth")
        self._11.addWidget(self.sbLineWidth)
        self.btLineWidth = QtWidgets.QPushButton(self.groupBox_7)
        self.btLineWidth.setMinimumSize(QtCore.QSize(60, 0))
        self.btLineWidth.setMaximumSize(QtCore.QSize(60, 16777215))
        self.btLineWidth.setObjectName("btLineWidth")
        self._11.addWidget(self.btLineWidth)
        self._8.addLayout(self._11)
        self._7.addLayout(self._8)
        self.line_3 = QtWidgets.QFrame(self.groupBox_7)
        self.line_3.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self._7.addWidget(self.line_3)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self._13 = QtWidgets.QHBoxLayout()
        self._13.setObjectName("_13")
        self.label_5 = QtWidgets.QLabel(self.groupBox_7)
        self.label_5.setObjectName("label_5")
        self._13.addWidget(self.label_5)
        self.teTimeMarker = QtWidgets.QTimeEdit(self.groupBox_7)
        self.teTimeMarker.setObjectName("teTimeMarker")
        self._13.addWidget(self.teTimeMarker)
        self.btTimeMarker = QtWidgets.QPushButton(self.groupBox_7)
        self.btTimeMarker.setMinimumSize(QtCore.QSize(60, 0))
        self.btTimeMarker.setMaximumSize(QtCore.QSize(60, 16777215))
        self.btTimeMarker.setObjectName("btTimeMarker")
        self._13.addWidget(self.btTimeMarker)
        self.verticalLayout_2.addLayout(self._13)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_6 = QtWidgets.QLabel(self.groupBox_7)
        self.label_6.setMinimumSize(QtCore.QSize(110, 0))
        self.label_6.setMaximumSize(QtCore.QSize(110, 16777215))
        self.label_6.setObjectName("label_6")
        self.horizontalLayout.addWidget(self.label_6)
        self.cbPlotInView = QtWidgets.QComboBox(self.groupBox_7)
        self.cbPlotInView.setMinimumSize(QtCore.QSize(150, 0))
        self.cbPlotInView.setObjectName("cbPlotInView")
        self.cbPlotInView.addItem("")
        self.horizontalLayout.addWidget(self.cbPlotInView)
        self.btPlotInView = QtWidgets.QPushButton(self.groupBox_7)
        self.btPlotInView.setMinimumSize(QtCore.QSize(60, 0))
        self.btPlotInView.setMaximumSize(QtCore.QSize(60, 16777215))
        self.btPlotInView.setObjectName("btPlotInView")
        self.horizontalLayout.addWidget(self.btPlotInView)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_7 = QtWidgets.QLabel(self.groupBox_7)
        self.label_7.setMinimumSize(QtCore.QSize(110, 0))
        self.label_7.setMaximumSize(QtCore.QSize(110, 16777215))
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_2.addWidget(self.label_7)
        self.cbRemoveFromView = QtWidgets.QComboBox(self.groupBox_7)
        self.cbRemoveFromView.setMinimumSize(QtCore.QSize(150, 0))
        self.cbRemoveFromView.setObjectName("cbRemoveFromView")
        self.cbRemoveFromView.addItem("")
        self.horizontalLayout_2.addWidget(self.cbRemoveFromView)
        self.btRemoveFromView = QtWidgets.QPushButton(self.groupBox_7)
        self.btRemoveFromView.setMinimumSize(QtCore.QSize(60, 0))
        self.btRemoveFromView.setMaximumSize(QtCore.QSize(60, 16777215))
        self.btRemoveFromView.setObjectName("btRemoveFromView")
        self.horizontalLayout_2.addWidget(self.btRemoveFromView)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self._7.addLayout(self.verticalLayout_2)
        self.verticalLayout_3.addWidget(self.groupBox_7)
        self.tabGUI.addTab(self.tabData, "")
        self.verticalLayout.addWidget(self.tabGUI)
        TrajectoriesWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(TrajectoriesWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 732, 26))
        self.menubar.setObjectName("menubar")
        self.menu_File = QtWidgets.QMenu(self.menubar)
        self.menu_File.setObjectName("menu_File")
        TrajectoriesWindow.setMenuBar(self.menubar)
        self.actionOpenTrajectories = QtWidgets.QAction(TrajectoriesWindow)
        self.actionOpenTrajectories.setObjectName("actionOpenTrajectories")
        self.actionOpenFlightTrack = QtWidgets.QAction(TrajectoriesWindow)
        self.actionOpenFlightTrack.setObjectName("actionOpenFlightTrack")
        self.action_Quit = QtWidgets.QAction(TrajectoriesWindow)
        self.action_Quit.setObjectName("action_Quit")
        self.actionCloseElement = QtWidgets.QAction(TrajectoriesWindow)
        self.actionCloseElement.setObjectName("actionCloseElement")
        self.menu_File.addAction(self.actionOpenTrajectories)
        self.menu_File.addAction(self.actionOpenFlightTrack)
        self.menu_File.addSeparator()
        self.menu_File.addAction(self.action_Quit)
        self.menubar.addAction(self.menu_File.menuAction())

        self.retranslateUi(TrajectoriesWindow)
        self.tabGUI.setCurrentIndex(0)
        self.cbSelectElements.setCurrentIndex(0)
        self.action_Quit.triggered.connect(TrajectoriesWindow.close)
        QtCore.QMetaObject.connectSlotsByName(TrajectoriesWindow)

    def retranslateUi(self, TrajectoriesWindow):
        _translate = QtCore.QCoreApplication.translate
        TrajectoriesWindow.setWindowTitle(_translate("TrajectoriesWindow", "Trajectories - Mission Support System"))
        self.groupBox.setTitle(_translate("TrajectoriesWindow", "Open trajectory items:"))
        self.label_4.setText(_translate("TrajectoriesWindow", "Select:"))
        self.leSelectionQuery.setToolTip(_translate("TrajectoriesWindow", "Enter selection criteria here. Available variables are %lat, %lon, %pres, \n"
"and %meta(\"NAME\"), where NAME has to be the name of a matatag that appears in the data. \n"
"For example, type \'%lat >= 45.\' will select all elements with a start point latitude larger \n"
"equal to 45 degrees after you click the button on the left. To select all trajectories with a \n"
"start latitude larger than 45 degrees at flight level 320, type \'%lat > 45. and %meta(\"flightlevel\") == 320\'."))
        self.cbSelectElements.setItemText(0, _translate("TrajectoriesWindow", "from all items"))
        self.cbSelectElements.setItemText(1, _translate("TrajectoriesWindow", "from children of current node"))
        self.btSelectMapElements.setToolTip(_translate("TrajectoriesWindow", "Select all items listed below that match the criteria given on the right."))
        self.btSelectMapElements.setText(_translate("TrajectoriesWindow", "&go!"))
        self.groupBox_7.setTitle(_translate("TrajectoriesWindow", "Modify selected items:"))
        self.label.setText(_translate("TrajectoriesWindow", "Colour:"))
        self.cbColour.setItemText(0, _translate("TrajectoriesWindow", "None"))
        self.cbColour.setItemText(1, _translate("TrajectoriesWindow", "blue"))
        self.cbColour.setItemText(2, _translate("TrajectoriesWindow", "green"))
        self.cbColour.setItemText(3, _translate("TrajectoriesWindow", "red"))
        self.cbColour.setItemText(4, _translate("TrajectoriesWindow", "cyan"))
        self.cbColour.setItemText(5, _translate("TrajectoriesWindow", "magenta"))
        self.cbColour.setItemText(6, _translate("TrajectoriesWindow", "yellow"))
        self.cbColour.setItemText(7, _translate("TrajectoriesWindow", "black"))
        self.cbColour.setItemText(8, _translate("TrajectoriesWindow", "white"))
        self.btColour.setToolTip(_translate("TrajectoriesWindow", "Set the colour of the selected items."))
        self.btColour.setText(_translate("TrajectoriesWindow", "apply"))
        self.label_2.setText(_translate("TrajectoriesWindow", "Line Style:"))
        self.cbLineStyle.setItemText(0, _translate("TrajectoriesWindow", "None"))
        self.cbLineStyle.setItemText(1, _translate("TrajectoriesWindow", "-"))
        self.cbLineStyle.setItemText(2, _translate("TrajectoriesWindow", "--"))
        self.cbLineStyle.setItemText(3, _translate("TrajectoriesWindow", "-."))
        self.cbLineStyle.setItemText(4, _translate("TrajectoriesWindow", ":"))
        self.btLineStyle.setToolTip(_translate("TrajectoriesWindow", "Set the line style of the selected items."))
        self.btLineStyle.setText(_translate("TrajectoriesWindow", "apply"))
        self.label_3.setText(_translate("TrajectoriesWindow", "Line Width:"))
        self.btLineWidth.setToolTip(_translate("TrajectoriesWindow", "Set the line width of the selected items."))
        self.btLineWidth.setText(_translate("TrajectoriesWindow", "apply"))
        self.label_5.setText(_translate("TrajectoriesWindow", "Draw time markers (hh:mm):"))
        self.teTimeMarker.setDisplayFormat(_translate("TrajectoriesWindow", "hh:mm"))
        self.btTimeMarker.setToolTip(_translate("TrajectoriesWindow", "Draw time markers along the trajectories. These will be dots with the spacing given on the right (hh:mm)."))
        self.btTimeMarker.setText(_translate("TrajectoriesWindow", "apply"))
        self.label_6.setText(_translate("TrajectoriesWindow", "Plot in view:"))
        self.cbPlotInView.setItemText(0, _translate("TrajectoriesWindow", "None"))
        self.btPlotInView.setText(_translate("TrajectoriesWindow", "apply"))
        self.label_7.setText(_translate("TrajectoriesWindow", "Remove from view:"))
        self.cbRemoveFromView.setItemText(0, _translate("TrajectoriesWindow", "None"))
        self.btRemoveFromView.setText(_translate("TrajectoriesWindow", "apply"))
        self.tabGUI.setTabText(self.tabGUI.indexOf(self.tabData), _translate("TrajectoriesWindow", "Data"))
        self.menu_File.setTitle(_translate("TrajectoriesWindow", "&File"))
        self.actionOpenTrajectories.setText(_translate("TrajectoriesWindow", "&Open Trajectories..."))
        self.actionOpenTrajectories.setShortcut(_translate("TrajectoriesWindow", "Ctrl+O"))
        self.actionOpenFlightTrack.setText(_translate("TrajectoriesWindow", "Open &Flight Track..."))
        self.actionOpenFlightTrack.setShortcut(_translate("TrajectoriesWindow", "Ctrl+F"))
        self.action_Quit.setText(_translate("TrajectoriesWindow", "&Exit"))
        self.action_Quit.setShortcut(_translate("TrajectoriesWindow", "Ctrl+X"))
        self.actionCloseElement.setText(_translate("TrajectoriesWindow", "&Close Element..."))




# -*- coding: utf-8 -*-
"""

    mslib.mscolab.models
    ~~~~~~~~~~~~~~~~~~~~

    sqlalchemy models for mscolab database

    This file is part of mss.

    :copyright: Copyright 2019 Shivashis Padhi
    :copyright: Copyright 2019-2020 by the mss team, see AUTHORS.
    :license: APACHE-2.0, see LICENSE for details.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from flask_sqlalchemy import SQLAlchemy
from passlib.apps import custom_app_context as pwd_context
from itsdangerous import (TimedJSONWebSignatureSerializer
                          as Serializer, BadSignature, SignatureExpired)
from mslib.mscolab.conf import mscolab_settings

import logging
import datetime

db = SQLAlchemy()


class User(db.Model):

    __tablename__ = 'users'
    id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    username = db.Column(db.String(255))
    emailid = db.Column(db.String(255), unique=True)
    password = db.Column(db.String(255), unique=True)

    def __init__(self, emailid, username, password):
        self.username = username
        self.emailid = emailid
        self.hash_password(password)

    def __repr__(self):
        return('<User %r>' % self.username)

    def hash_password(self, password):
        self.password = pwd_context.encrypt(password)

    def verify_password(self, password_):
        return pwd_context.verify(password_, self.password)

    def generate_auth_token(self, expiration=864000):
        s = Serializer(mscolab_settings.SECRET_KEY, expires_in=expiration)
        return s.dumps({'id': self.id})

    @staticmethod
    def verify_auth_token(token):
        """
        token is the authentication string provided by client for each request
        """
        s = Serializer(mscolab_settings.SECRET_KEY)
        try:
            data = s.loads(token)
        except SignatureExpired:
            logging.debug("Signature Expired")
            return None  # valid token, but expired
        except BadSignature:
            logging.debug("Bad Signature")
            return None  # invalid token
        user = User.query.filter_by(id=data['id']).first()
        return user


class Connection(db.Model):

    __tablename__ = 'connections'
    id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    s_id = db.Column(db.String(255), unique=True)
    u_id = db.Column(db.Integer, db.ForeignKey('users.id'))

    def __init__(self, u_id, s_id):
        self.u_id = u_id
        self.s_id = s_id

    def __repr__(self):
        return('<Connection %s %s>'.format(self.s_id, self.u_id))


class Permission(db.Model):

    __tablename__ = 'permissions'
    id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    p_id = db.Column(db.Integer, db.ForeignKey('projects.id'))
    u_id = db.Column(db.Integer, db.ForeignKey('users.id'))
    access_level = db.Column(db.Enum("admin", "collaborator", "viewer", "creator", name="access_level"))

    def __init__(self, u_id, p_id, access_level):
        """
        u_id: user-id
        p_id: process-id
        access_level: the type of authorization to the project
        """
        self.u_id = u_id
        self.p_id = p_id
        self.access_level = access_level

    def __repr__(self):
        return('<Permission user %s project %s access level %s>'.format(self.u_id, self.p_id, str(self.access_level)))


class Project(db.Model):

    __tablename__ = "projects"
    id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    path = db.Column(db.String(255), unique=True)
    description = db.Column(db.String(255))
    autosave = db.Column(db.Boolean)

    def __init__(self, path, description, autosave):
        """
        path: path to the project
        description: small description of project,
        autosave: a boolean to show if autosave is enabled or disabled
            for the project
        """
        self.path = path
        self.description = description
        self.autosave = autosave

    def __repr__(self):
        return('<Project path %s desc %s>'.format(self.path, self.description))


class Message(db.Model):

    __tablename__ = "messages"
    id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    p_id = db.Column(db.Integer, db.ForeignKey('projects.id'))
    u_id = db.Column(db.Integer, db.ForeignKey('users.id'))
    text = db.Column(db.Text)
    created_at = db.Column(db.DateTime, default=datetime.datetime.utcnow)

    def __init__(self, p_id, u_id, text):
        self.p_id = p_id
        self.u_id = u_id
        self.text = text

    def __repr__(self):
        return('<Message %s user %s in %s>'.format(self.text, self.u_id, self.p_id))


class Change(db.Model):

    __tablename__ = "changes"
    id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    p_id = db.Column(db.Integer, db.ForeignKey('projects.id'))
    u_id = db.Column(db.Integer, db.ForeignKey('users.id'))
    content = db.Column(db.TEXT)
    comment = db.Column(db.String(255), default=None)
    commit_hash = db.Column(db.String(255), default=None)
    created_at = db.Column(db.DateTime, default=datetime.datetime.utcnow)

    def __init__(self, p_id, u_id, content, commit_hash, comment=""):
        self.p_id = p_id
        self.u_id = u_id
        self.content = content
        self.commit_hash = commit_hash
        self.comment = comment



# -*- coding: utf-8 -*-
"""

    mslib.mscolab.seed
    ~~~~~~~~~~~~~~~~~~~~

    Seeder utility for database

    This file is part of mss.

    :copyright: Copyright 2019 Shivashis Padhi
    :copyright: Copyright 2019-2020 by the mss team, see AUTHORS.
    :license: APACHE-2.0, see LICENSE for details.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from flask import Flask

from mslib.mscolab.models import User, db, Permission, Project
from mslib.mscolab.conf import mscolab_settings
# set the project root directory as the static folder
app = Flask(__name__, static_url_path='')


def seed_data(db_uri):

    app.config['SQLALCHEMY_DATABASE_URI'] = db_uri
    app.config['SECRET_KEY'] = mscolab_settings.SECRET_KEY
    db.init_app(app)

    with app.app_context():
        # create users
        users = [{
            'username': 'a',
            'id': 8,
            'password': 'a',
            'emailid': 'a'
        }, {
            'username': 'b',
            'id': 9,
            'password': 'b',
            'emailid': 'b'
        }, {
            'username': 'c',
            'id': 10,
            'password': 'c',
            'emailid': 'c'
        }]
        for user in users:
            db_user = User(user['emailid'], user['username'], user['password'])
            db_user.id = user['id']
            db.session.add(db_user)
        db.session.commit()

        # create projects
        projects = [{
            'id': 1,
            'path': 'one',
            'description': 'a, b',
            'autosave': False
        }, {
            'id': 2,
            'path': 'two',
            'description': 'b, c',
            'autosave': False
        }, {
            'id': 3,
            'path': 'three`',
            'description': 'a, c',
            'autosave': False
        }]
        for project in projects:
            db_project = Project(project['path'], project['description'], project['autosave'])
            db_project.id = project['id']
            db.session.add(db_project)
        db.session.commit()

        # create permissions
        permissions = [{
            'u_id': 8,
            'p_id': 1,
            'access_level': "creator"
        }, {
            'u_id': 9,
            'p_id': 1,
            'access_level': "collaborator"
        }, {
            'u_id': 9,
            'p_id': 2,
            'access_level': "creator"
        }, {
            'u_id': 10,
            'p_id': 2,
            'access_level': "collaborator"
        }, {
            'u_id': 10,
            'p_id': 3,
            'access_level': "creator"
        }, {
            'u_id': 8,
            'p_id': 3,
            'access_level': "collaborator"
        }, {
            'u_id': 10,
            'p_id': 1,
            'access_level': "viewer"
        }]
        for perm in permissions:
            db_perm = Permission(perm['u_id'], perm['p_id'], perm['access_level'])
            db.session.add(db_perm)
        db.session.commit()
        db.session.close()


def create_tables(db_uri):
    app.config['SQLALCHEMY_DATABASE_URI'] = db_uri
    app.config['SECRET_KEY'] = mscolab_settings.SECRET_KEY
    db.init_app(app)
    with app.app_context():
        db.create_all()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui_timeseriesview_window.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_TimeSeriesViewWindow(object):
    def setupUi(self, TimeSeriesViewWindow):
        TimeSeriesViewWindow.setObjectName("TimeSeriesViewWindow")
        TimeSeriesViewWindow.resize(648, 738)
        self.centralwidget = QtWidgets.QWidget(TimeSeriesViewWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.mpl = MplTimeSeriesViewWidget(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.mpl.sizePolicy().hasHeightForWidth())
        self.mpl.setSizePolicy(sizePolicy)
        self.mpl.setCursor(QtGui.QCursor(QtCore.Qt.CrossCursor))
        self.mpl.setObjectName("mpl")
        self.horizontalLayout.addWidget(self.mpl)
        TimeSeriesViewWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(TimeSeriesViewWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 648, 26))
        self.menubar.setObjectName("menubar")
        TimeSeriesViewWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(TimeSeriesViewWindow)
        self.statusbar.setObjectName("statusbar")
        TimeSeriesViewWindow.setStatusBar(self.statusbar)

        self.retranslateUi(TimeSeriesViewWindow)
        QtCore.QMetaObject.connectSlotsByName(TimeSeriesViewWindow)

    def retranslateUi(self, TimeSeriesViewWindow):
        _translate = QtCore.QCoreApplication.translate
        TimeSeriesViewWindow.setWindowTitle(_translate("TimeSeriesViewWindow", "Time Series View - Mission Support System"))

from mslib.msui.mpl_qtwidget import MplTimeSeriesViewWidget
